;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xf13b2d32f73239e70692ef59c6deac30 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6eb28f9f6dc63a1ffb6e28eee3cb2156 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2d007aea93684830d65cc4f5d4b8fa85 . "./code.cg2js/cg-toolbar.fasl")
(#xd9e2c8fa3c6cb6d3d31eaa51d8601db3 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x01bf63c1a9a833eb8dc3a67b784f8518 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x7205674732d96b3dda859dd81d1c960f . "./code.cg2js/cg-class-grid.fasl")
(#xe754cc1c64557a75a662980ba5c08a9f . "./code.cg2js/cg-tray.fasl")
(#x026683f2e49a732d3dae6a3467080a97 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x8069e78a95c74ae22fcb9038a8a179c1 . "./code.cg2js/cg-class-support.fasl")
(#x55488c421615653bcdd0c5470885f1a9 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xd0a8b05fab55aad80743ee2e92356fb3 . "./code.cg2js/cg-dropping-outline.fasl")
(#xff7e8202c39697badcac44d5540e1d6f . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xad64d73d7172748103c68bbef92d732b . "./code.cg2js/cg-choice-list.fasl")
(#x981650fce43605e2e9f92c006eb2f20c . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xdda6e8245fdbf65c909b9509bf018534 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x9b3172c37ca242b75015c0832b1f77de . "./code.cg2js/cg-timer.fasl")
(#x48138a809c1058416f94b2ee9caff01b . "./code.cg2js/cg-directory-dialog.fasl")
(#x0f293fa94999c3f7205162d92d10a5a5 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x070006f1b196486ff4396103ba72cb67 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd59b665cfdc6b5156182f10a2a2c3d76 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xe6012fa0e1b5183b37263cdc83aa5326 . "./code.cg2js/cg-html-widget.fasl")
(#xf87985fb86060ba10aece55d0624cf89 . "./code.cg2js/cg-selected-object.fasl")
(#x71b9c9cb630be99d6464fcdf3c6421b1 . "./code.cg2js/cg-lamp.fasl")
(#x2e04455f130adce49217a2068e031a4b . "./code.cg2js/cg-scroll-bar.fasl")
(#xc3aff150296f17e1289b1a7bedb2f3d3 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x328f64b1b0871f51160cecc46531776d . "./code.cg2js/cg-message-dialog.fasl")
(#x2dc99d9a7f65675469acfc267fc0d990 . "./code.cg2js/cg-dde-utils.fasl")
(#x56fea734bbd7681eb451e1e10793dbaf . "./code.cg2js/cg-group-box.fasl")
(#x5dc87c1328696420b2fa808ccc5a4dbd . "./code.cg2js/cg-find-string-dialog.fasl")
(#x6e4fd0bbdb15c7512bf076208544435f . "./code.cg2js/cg-split-bar.fasl")
(#x9a20a0d8751fc50e8814cd80351fa82d . "./code.cg2js/cg-radio-button.fasl")
(#x86ee132b8c6d526232d7b2bccbcfc5f9 . "./code.cg2js/cg-base.fasl")
(#xa076ecefcf52bddb1557a103fb072d5a . "./code.cg2js/cg-paren-matching.fasl")
(#xb70d350595ac79ad045a8a550eeea035 . "./code.cg2js/cg-graphics-context.fasl")
(#x54506cfd4bc9788f6be4c6ed95e8c4a1 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x5cc01ead451d073502e2a8275161665b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x94959d8c2f39a27897f352919a0732cc . "./code.cg2js/cg-plot-widget.fasl")
(#xdce28989119acdc5939bd2759ebdb6d0 . "./code.cg2js/cg-dev.fasl")
(#x0af9de08d4314d591db970ba6ad1abd8 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf9a3178bb6fe95ecb845192487b66261 . "./code.cg2js/cg-tab-control.fasl")
(#xa4360c9018093523b24535a9aea75a66 . "./code.cg2js/cg-fill-texture.fasl")
(#xf46c7b3a279d98125ea8eac4cbbe89a9 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xcfdda908225f1db07ab7a4ba6ea8a915 . "./code.cg2js/cg-trackbar.fasl")
(#x9f99cc8f8adfcf2d326da5da9aea4d98 . "./code.cg2js/cg-nodes.fasl")
(#x2a41e2e607787478cbbd086ee416a8bd . "./code.cg2js/cg-common-control.fasl")
(#x91bd54707b2a1a8b436a4adc6a04afc6 . "./code.cg2js/cg-list-view.fasl")
(#x2be1c4d8b8959f6035c8d2730bd2d3a2 . "./code.cg2js/cg-rich-edit.fasl")
(#x7e20a8fb9008bfe8391c25bbbe609fa2 . "./code.cg2js/cg-project-window.fasl")
(#xf410d01ca6cdc25503b1c571e12fc5c2 . "./code.cg2js/cg-item-list.fasl")
(#x00d74c5d0871ec05cf48b94b732cf774 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4a62ea431f2339b5f3ea8711de147809 . "./code.cg2js/cg-get-position.fasl")
(#xf140e0a4b187b8d322e96084d77c3d9d . "./code.cg2js/cg-bitmap-pane.fasl")
(#x5dff654d5a9a052e5903c4413a159c71 . "./code.cg2js/cg-drawable.fasl")
(#x8ca4b58cf8c580e4894e4f0d31c3030b . "./code.cg2js/cg-lettered-menu.fasl")
(#x842b938af84edb610c4914fd8fbd5d62 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xee9f89bb01d283cf98e5d57e6aec7c6d . "./code.cg2js/cg-html-dialog.fasl")
(#x49c945bd7bd70219f1ff84ed5c23bc9a . "./code.cg2js/cg-object-editor-layout.fasl")
(#xfea99efa01cd09daff8a481ad422273e . "./code.cg2js/cg-utility-dialog.fasl")
(#xb2e218ab72ee71e5c65e8ec394ea5822 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x87a3d1b7f2145c28ed79ab08a9e70264 . "./code.cg2js/cg-property.fasl")
(#xb819c1eeb53c3abfc87b0bed5b4d44ef . "./code.cg2js/cg-menu.fasl")
(#x1b6f03b98d6d277738a122efc33f3594 . "./code.cg2js/cg-clipboard.fasl")
(#x47bef72edbcb5eddd4cea027029ab8ce . "./code.cg2js/cg-toggling-widget.fasl")
(#xe83c07aa6b0acd8458d06a1cc05629cc . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x09d37c7789703feea2078fd87567e945 . "./code.cg2js/cg-template-string.fasl")
(#x04db9132e0da143c431429fc74cb93d7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x890691ceda9cb73b3d2e80f3b1cb8145 . "./code.cg2js/cg-os-window.fasl")
(#xf685ead3fbc5a173f3ce708e3a857ab4 . "./code.cg2js/cg-grid-widget.fasl")
(#x78c7ac3c5360b2b1a396504ec76a44fe . "./code.cg2js/cg-picture-widget.fasl")
(#xc4cf9945a1547bdcef3b0ca9b7296dbf . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x76a5843364c79f7cd607e0e84880b911 . "./code.cg2js/cg-static-text.fasl")
(#x5260a7520f3705c1368d360fca93a67c . "./code.cg2js/cg-clipboard-stack.fasl")
(#x469c3f94e1a1850bdec0d17d3de09759 . "./code.cg2js/cg-text-widget.fasl")
(#x888966c0f12fba2ec78467ac275dcc36 . "./code.cg2js/cg-calendar.fasl")
(#x938da22613c6153209c083d7cfefb582 . "./code.cg2js/cg-check-box.fasl")
(#x21ed75dbb1de28984e90661a7156e087 . "./code.cg2js/cg-acache.fasl")
(#x65662c94338136900a28af57bad03207 . "./code.cg2js/cg-wrap-string.fasl")
(#x090f225d76a0055a38c1ce460ce5a09b . "./code.cg2js/cg-bitmap-stream.fasl")
(#xa1835965fdc24c115c8c07f38106ad78 . "./code.cg2js/cg-font-dialog.fasl")
(#xb27e7fde19192baa22f5cceb91892062 . "./code.cg2js/cg-outline.fasl")
(#xdbc1fc5bc44e7bd740471e2f0e224d4d . "./code.cg2js/cg-shortcut-menu.fasl")
(#x08aa15f5d4767f4dd49563566e83d9a6 . "./code.cg2js/cg-status-bar.fasl")
(#x52edfd1c7142853282b444ea996add89 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8512f1f05297092c01a5f24eed92cb3a . "./code.cg2js/cg-hotspot.fasl")
(#x99b5d5b0ed8a93a29adac0dff2622d71 . "./code.cg2js/cg-scaling-stream.fasl")
(#xcf2a20280f5da2de508f3741a37a309d . "./code.cg2js/cg-icon-pixmap.fasl")
(#x70788a81dd283b00be9bf4fd4e64c609 . "./code.cg2js/cg-file-dialog.fasl")
(#xb1b825483cab8d459d721c805d2862a5 . "./code.cg2js/cg-color-dialog.fasl")
(#x3c516771f64975c6880ca1529fdb5209 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7f20354c9120161e1f0328a9ec12036e . "./code.cg2js/cg-object-editor.fasl")
(#x5c09894d4bec5144b22453a065bf5fac . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x578025121e9ae72208e98e71612517f1 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x5f8e7e498e1a4006611e4ba8ad59df59 . "./code.cg2js/cg-pixmap.fasl")
(#x1becebfd58fd9aad4ddc713e83c3a996 . "./code.cg2js/cg-intersect.fasl")
(#x237cc72fde7b52e9829f83dcc747f80b . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x13198d72340496c3aadb6ad1d2ee315c . "./code.cg2js/cg-icon.fasl")
(#x369be82dfb6b2eafe929273250eee099 . "./code.cg2js/cg-yes-no-string.fasl")
(#xa9ceb252259e654063935f6c52fb6bcc . "./code.cg2js/cg-text-or-combo.fasl")
(#x085b4d19cba15677b852eec6b992261b . "./code.cg2js/cg.fasl")
(#xaae3e54111655a623c2cf5412780e183 . "./code.cg2js/cg-dialog-item.fasl")
(#xcc514f4a1a016f37e22efbb0988c78c1 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x57c01c518b103a93b98117ac34281e21 . "./code.cg2js/cg-caret.fasl")
(#x6c00524a839cb1a21ca341999a0e6c86 . "./code.cg2js/cg-editable-text.fasl")
(#xee0ef14a17715db1f17f958c3ca23176 . "./code.cg2js/cg-choose-printer.fasl")
(#x09e19b2e9aaa6528ae9f68b541aec4a8 . "./code.cg2js/cg-yes-no-list.fasl")
(#x9e0d9e89ee085f152cd5c3823d449b78 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x26528e0ea5cd69a81819eab66a48a133 . "./code.cg2js/cg-tooltip.fasl")
(#x735dea9728b6c6e62f75d5b5dd60e15c . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3d9627b1d6bf64413b03eb8a95f51292 . "./code.cg2js/cg-combo-box.fasl")
(#x7fe83b52e4ddd7c30a99aa28b102159c . "./code.cg2js/cg-curve.fasl")
(#x5f7661edf6c8fefd7c8fb80dab353ee0 . "./code.cg2js/cg-web-browser.fasl")
(#x0ba9f91c70100fdc5c235120966de3ef . "./code.cg2js/cg-printing.fasl")
(#x64fdea2926d17d67388a234e02d55812 . "./code.cg2js/cg-lisp-widget.fasl")
(#xbc749d8aa1c67b77973d3d365718d671 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x99ad8210a88688f9d8a23e67db8586fb . "./code.cg2js/cg-palette.fasl")
(#xb376570b4d9f725569c1af6ea8b21540 . "./code.cg2js/cg-os-widget.fasl")
(#x7544e08d58aef91f3f9d56be64938dd1 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x5184a1543a7e58904360e0b73f991f77 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xb1a1303afa6969e291a1fe36cef2afe4 . "./code.cg2js/cg-button.fasl")
(#xc3d049706f7ffd8cadb6d2c8b8ee4775 . "./code.cg2js/cg-edit-in-place.fasl")
(#x7ed3143febca3e0ec635fdd259671930 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbbcebf1bebe1588f2894bfb039bf7a03 . "./code.cg2js/cg-comtab.fasl")
(#x7ffc90b41ffd0356e0acf5fe4c4b6cc1 . "./code.cg2js/cg-string-dialog.fasl")
(#xfdb13a5bd1513bd713ef8a8dc0d05227 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xa5ec508f1a1bdd39b4d3bb9de78b483e . "./code.cg2js/cg-up-down-control.fasl")
(#x8e90bc7f3dbbbd9c695c1611eb4e76bb . "./code.cg2js/cg-layout.fasl")
(#x367db1e3e364232819a317f5512d51e1 . "./code.cg2js/cg-chart-widget.fasl")
(#x97b78dd342470cc468c4485ed8482ddf . "./code.cg2js/cg-lisp-text.fasl")
(#xa987ac53a2369a29991a6619f129b538 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x7d0f9c6589484433cc9f61bdebf92566 . "./code.cg2js/cg-progress-indicator.fasl")
(#xd7861bae967e2f3000148de257aa7de6 . "./code.idejs/ide-inspector.fasl")
(#xeb51c9761c1b3172fe3525b1eb6633ee . "./code.idejs/ide-grapher.fasl")
(#x6bc4fe06c05ee272dcb63672058f4ca5 . "./code.idejs/ide-base.fasl")
(#x57278ddc8fb1bf1fa5c83d6288f9a316 . "./code.idejs/ide-keyshort.fasl")
(#x45391fac0f581ce3d95caa8a0d89eccc . "./code.idejs/ide-dev.fasl")
(#x992c211ff8e0462289703e6ddee9ffd3 . "./code.idejs/ide-class-browser.fasl")
(#x84b1a8825740766bd39dd8e9ab21073b . "./code.idejs/ide-package-list.fasl")
(#x3f08774536988febee8cd133cd530223 . "./code.idejs/ide-intro.fasl")
(#x9589a7ab5ed753a7e035addfeb15ecd3 . "./code.idejs/ide-class-grapher.fasl")
(#x912365c03cb979aa1f28f91bd6be78e2 . "./code.idejs/ide-window-list.fasl")
(#x9c4a996c8953d66f266c84d50a1d0acc . "./code.idejs/ide-find-definitions.fasl")
(#xab12f5ba3d660baba4b3f81fb4645b6b . "./code.idejs/ide-trace.fasl")
(#x03934952e9cf9374b46411b60b8fbeea . "./code.idejs/ide-editor.fasl")
(#x4fbd359d8d737a522f276c9fd3218cca . "./code.idejs/ide-patch-dialog.fasl")
(#x1762a989b52d3944f81d5c2e92dcad1f . "./code.idejs/ide-project.fasl")
(#x6caa8cfca59e05f015f6d2a398e76c10 . "./code.idejs/ide.fasl")
(#x8e6f5c4a384d7009a2f036856144bc86 . "./code.idejs/ide-apropos.fasl")
(#x38a51e56ff75d92a87c3ed4e6c489cd2 . "./code.idejs/ide-acache.fasl")
(#x03541fab92c7e3862969dad4bf49f271 . "./code.idejs/ide-clipboard.fasl")
(#xbacec047d3f7bf74106a48981582fd0c . "./code.idejs/ide-process-browser.fasl")
(#x76c0cbe56fcba0d1e9a7c6fe642ddb7e . "./code.idejs/ide-find-in-files.fasl")
(#x62db7a9fc3c3fff5826be90deffe21c1 . "./code.idejs/ide-form.fasl")
(#x25e541c7abd042da8719fcbd5873658b . "./code.idejs/ide-navigator.fasl")
(#x49a17b9a618997e974ff2e9af0f2eb06 . "./cgjs.html")
(#x44f169492307df5920f609d11cc1a34c . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
